/*
 * IRC - Internet Relay Chat, ircd/m_sahost.c
 * Copyright (C) 1990 Jarkko Oikarinen and
 *                    University of Oulu, Computing Center
 *
 * See file AUTHORS in IRC package for additional names of
 * the programmers.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: m_sahost.c,v 1.1 2008/01/19 13:56:46 key2peace Exp $
 */

#include "channel.h"
#include "config.h"
#include "client.h"
#include "handlers.h"
#include "hash.h"
#include "ircd.h"
#include "ircd_chattr.h"
#include "ircd_defs.h"
#include "ircd_features.h"
#include "ircd_relay.h"
#include "ircd_reply.h"
#include "ircd_string.h"
#include "ircd_snprintf.h"
#include "ircd_alloc.h"
#include "list.h"
#include "msg.h"
#include "numeric.h"
#include "numnicks.h"
#include "querycmds.h"
#include "send.h"
#include "s_conf.h"
#include "s_user.h"
#include "s_debug.h"
#include "userload.h"
#include "patchlevel.h"

#include <string.h>
#include <ctype.h>
#include <stdlib.h>


int m_sahost(struct Client* cptr, struct Client* sptr, int parc, char* parv[])
{
  struct Client *acptr;
  char *s;
  int legalhost=1;

  if(parc<3)
    return(need_more_params(sptr, "SAHOST"));

  if (!HasPriv(sptr, PRIV_SAHOST))
    return send_reply(sptr, ERR_NOPRIVILEGES);

  /* Ignore SAHOST for a user that has quit */
  if(!(acptr = FindClient(parv[1])))
    return send_reply(sptr, ERR_SACOM, "SAHOST", "No such nick");

  if (IsChannelService(acptr))
    return send_reply(sptr, ERR_NOPRIVILEGES);

  /*
   * Check that the given real name is not greater
   * than HOSTLEN.
   */
  if (strlen(parv[2]) > HOSTLEN)
    return send_reply(sptr, ERR_SACOM, "SAHOST", "Host too long");

  for (s = parv[2]; *s; s++)
  {
    if (!IsHostChar(*s))
    {
      legalhost = 0;
      break;
    }
  }

  if (legalhost == 0)
  {
    return send_reply(sptr, ERR_SACOM, "SAHOST", "Illegal characters in hostname");
  }

  ircd_strncpy(cli_sockhost(acptr), parv[2], HOSTLEN);
  ircd_strncpy(cli_user(acptr)->host, parv[2], HOSTLEN);

  if (!IsServer(cptr)) {
    sendto_opmask_butone(0, SNO_OLDSNO, "%s used SAHOST on %s changing hostname to :%s", cli_name(cptr), parv[1], parv[2]);
    sendcmdto_serv_butone(sptr, CMD_SAHOST, cptr, "%s %s", parv[1], parv[parc - 1]);
  } else {
    sendto_opmask_butone(0, SNO_OLDSNO, "%s used SAHOST on %s changing hostname to :%s", cli_name(sptr), parv[1], parv[2]);
    sendcmdto_serv_butone(sptr, CMD_SAHOST, cptr, "%s %s", parv[1], parv[parc - 1]);
  }
  return 0;
}
